local tablet = AceLibrary("Tablet-2.0")

FuBar_ItemListFu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0")
FuBar_ItemListFu.hasIcon = "Interface\\Icons\\INV_Misc_QuestionMark"
FuBar_ItemListFu.clickableTooltip = true
FuBar_ItemListFu:RegisterDB("FuBar_ItemListFuDB", "FuBar_ItemListFuDBPC")
FuBar_ItemListFu:RegisterDefaults("char", {
	data = {
		{
			name = "Uncategorized",
			collapsed = false,
			items = {}
		}
	}
})

FuBar_ItemListFu.catddtables = {}
FuBar_ItemListFu.itemddtables = {}

FuBar_ItemListFu.dropdown = {
	type = "group", args = {
		cats = {
			type = "group",
			name = "Categories",
			desc = "List of all categories",
			args = FuBar_ItemListFu.catddtables,
			order = 1
		},
		newcat = {
			type = "text",
			name = "New Categorie",
			desc = "Adds a new categorie",
			usage = "<cat name>",
			get = false,
			set = function(val) FuBar_ItemListFu:AddCat(val) end,
			order = 2
		}
	}
}

FuBar_ItemListFu.OnMenuRequest = FuBar_ItemListFu.dropdown

function FuBar_ItemListFu:OnInitialize()
	self.itemdata = {}
end

function FuBar_ItemListFu:OnEnable()
	while #self.catddtables > 0 do table.remove(self.catddtables) end
	while #self.itemddtables > 0 do table.remove(self.itemddtables) end
	local catddtable, itemddtable
	for catindex, catdata in ipairs(self.db.char.data) do
		catddtable = self:GetCatDropdownTable()
		table.insert(self.itemddtables, {})
		catddtable.args.items.args = self.itemddtables[catindex]
		self:UpdateCatDropdownTable(catddtable, catindex)
		table.insert(self.catddtables, catddtable)
		for itemindex, item in ipairs(catdata.items) do
			self:UpdateItem(item)
			itemddtable = self:GetItemDropdownTable()
			self:UpdateItemDropdownTable(itemddtable, catindex, itemindex)
			table.insert(self.itemddtables[catindex], itemddtable)
		end
	end
end

function FuBar_ItemListFu:OnDataUpdate()
	for _, catdata in ipairs(self.db.char.data) do
		for _, item in ipairs(catdata.items) do
			self:UpdateItem(item)
		end
	end
end

function FuBar_ItemListFu:OnTextUpdate()
	if self.db.char.textitem and self.itemdata[self.db.char.textitem].name then
		self:SetIcon(self.itemdata[self.db.char.textitem].icon or "Interface\\Icons\\INV_Misc_QuestionMark")
		self:SetText("|cff00ff00"..self.itemdata[self.db.char.textitem].name..":|r |cffffff00"..self.itemdata[self.db.char.textitem].count.."|r(+"..self.itemdata[self.db.char.textitem].countbank..")")
	else
		self:SetIcon("Interface\\Icons\\INV_Misc_QuestionMark")
		self:SetText("|cffffffffItemList|cff00ff00Fu|r")
	end
end

function FuBar_ItemListFu:OnTooltipUpdate()
	local cat = tablet:AddCategory("columns", 3, "hideBlankLine", true, "showWithoutChildren", true)
	cat:AddLine({	text = "Item",	justify = "CENTER",
					text2 = "Bags",	justify2 = "CENTER",
					text3 = "Bank",	justify3 = "CENTER"})

	for catindex, catdata in ipairs(self.db.char.data) do
		cat:AddLine({	text = catdata.name,	justify = "LEFT",
						text2 = "",
						text3 = "",
						hasCheck = true,		checked = true,		checkIcon = catdata.collapsed and "Interface\\Buttons\\UI-PlusButton-Up" or "Interface\\Buttons\\UI-MinusButton-Up",
						func = "OnClick_TooltipCat",	arg1 = self,	arg2 = catindex})
		if not catdata.collapsed then
			for itemindex, item in ipairs(catdata.items) do
				if self.itemdata[item].name then
					cat:AddLine({	text = self.itemdata[item].name,		justify = "LEFT",
									textR = 1, textG = 1, textB = 0,
									text2 = self.itemdata[item].count,		justify2 = "RIGHT",
									text2R = 1, text2G = 1, text2B = 1,
									text3 = self.itemdata[item].countbank,	justify3 = "RIGHT",
									text3R = 1, text3G = 1, text3B = 1,
									hasCheck = true,	checked = true,		checkIcon = self.itemdata[item].icon,
									func = "OnClick_TooltipItem",	arg1 = self,	arg2 = catindex, 	arg3 = itemindex})

				else
					cat:AddLine({	text = "Item "..item,					justify = "LEFT",
									textR = 1, textG = 1, textB = 0,
									text2 = self.itemdata[item].count,		justify2 = "RIGHT",
									text2R = 1, text2G = 1, text2B = 1,
									text3 = self.itemdata[item].countbank,	justify3 = "RIGHT",
									text3R = 1, text3G = 1, text3B = 1,
									hasCheck = true,	checked = true,		checkIcon = "Interface\\Icons\\INV_Misc_QuestionMark",
									func = "OnClick_TooltipItem",	arg1 = self,	arg2 = catindex, 	arg3 = itemindex})
				end
			end
		end
	end
end

function FuBar_ItemListFu:UpdateItem(item)
	if not self.itemdata[item] then self.itemdata[item] = {} end
	self.itemdata[item].count = GetItemCount(item)
	self.itemdata[item].countbank = GetItemCount(item, true) - self.itemdata[item].count
	if not self.itemdata[item].name then
--		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(item)
		local itemName, itemLink, _, _, _, _, _, _, _, itemTexture = GetItemInfo(item)
		self.itemdata[item].name = itemName
		self.itemdata[item].link = itemLink
		self.itemdata[item].icon = itemTexture
	end
end

function FuBar_ItemListFu:OnClick_TooltipCat(catindex)
	if IsControlKeyDown() then
	else
		self.db.char.data[catindex].collapsed = not self.db.char.data[catindex].collapsed
	end
end

function FuBar_ItemListFu:OnClick_TooltipItem(catindex, itemindex)
	if IsControlKeyDown() then
	elseif IsShiftKeyDown() then
		if self.itemdata[self.db.char.data[catindex].items[itemindex]].link then
			ChatEdit_InsertLink(self.itemdata[self.db.char.data[catindex].items[itemindex]].link)
		end
	else
		if self.db.char.textitem == self.db.char.data[catindex].items[itemindex] then
			self.db.char.textitem = nil
		else
			self.db.char.textitem = self.db.char.data[catindex].items[itemindex]
		end
		self:UpdateText()
	end
end

function FuBar_ItemListFu:AddCat(catname)
	if not catname or catname == "" then return end
	for _, catdata in ipairs(self.db.char.data) do
		if catdata.name == catname then return end
	end
	local cattable = {
		name = catname,
		collapsed = false,
		items = {}
	}
	table.insert(self.db.char.data, cattable)
	table.insert(self.itemddtables, {})
	local catddtable = self:GetCatDropdownTable()
	catddtable.args.items.args = self.itemddtables[#self.db.char.data]
	self:UpdateCatDropdownTable(catddtable, #self.db.char.data)
	table.insert(self.catddtables, catddtable)
	self:UpdateTooltip()
end

function FuBar_ItemListFu:AddItem(item, catindex)
	for _, catdata in ipairs(self.db.char.data) do
		for _, itemid in ipairs(catdata.items) do
			if itemid == item then return end
		end
	end
	self:UpdateItem(item)
	if not catindex then catindex = 1 end
	table.insert(self.db.char.data[catindex].items, item)
	local itemddtable = self:GetItemDropdownTable()
	self:UpdateItemDropdownTable(itemddtable, catindex, #self.db.char.data[catindex].items)
	table.insert(self.itemddtables[catindex], itemddtable)
	self:UpdateTooltip()
end

function FuBar_ItemListFu:GetCatDropdownTable()
	local catddtable = {
		type = "group",
		args = {
			items = {
				type = "group",
				name = "Items",
				desc = "Items in this categorie",
				order = 1
			},
			name = {
				type = "text",
				name = "Name",
				desc = "Change the categorie's name",
				usage = "<cat name>",
				order = 2
			},
			order = {
				type = "range",
				name = "Position",
				desc = "Change the categorie's position",
				min = 1,
				step = 1,
				order = 3
			},
			remove = {
				type = "execute",
				name = "Remove",
				desc = "Removes the categorie",
				order = 4
			}
		}
	}
	return catddtable
end

function FuBar_ItemListFu:UpdateCatDropdownTable(catddtable, catindex)
	catddtable.name = self.db.char.data[catindex].name
	catddtable.desc = string.format("Settings for categorie %s", catddtable.name)
	catddtable.order = catindex
	catddtable.args.name.get = function() return catddtable.name end
	catddtable.args.name.set = function(val) FuBar_ItemListFu:ChangeCatName(catindex, val) end
	catddtable.args.order.min = #self.db.char.data <= 1 and 0 or 1
	catddtable.args.order.max = #self.db.char.data
	catddtable.args.order.get = function() return catindex end
	catddtable.args.order.set = function(val) FuBar_ItemListFu:ChangeCatPos(catindex, val) end
	catddtable.args.order.disabled = #self.db.char.data <= 1
	catddtable.args.remove.func = function() FuBar_ItemListFu:RemoveCat(catindex) end
end

function FuBar_ItemListFu:ChangeCatName(catindex, name)
	for _, catdata in ipairs(self.db.char.data) do
		if catdata.name == name then return end
	end
	self.db.char.data[catindex].name = name
	self:UpdateCatDropdownTable(self.catddtables[catindex], catindex)
	for catid, cats in ipairs(self.itemddtables) do
		for itemindex, itemddtable in ipairs(cats) do
			self:UpdateItemDropdownTableCats(itemddtable, catid, itemindex)
		end
	end
	self:UpdateTooltip()
end

function FuBar_ItemListFu:ChangeCatPos(catindex, pos)
	if pos == catindex then return end
	local catdata = table.remove(self.db.char.data, catindex)
	table.insert(self.db.char.data, pos, catdata)
	local catddtable = table.remove(self.catddtables, catindex)
	table.insert(self.catddtables, pos, catddtable)
	for i = catindex, pos do
		self:UpdateCatDropdownTable(self.catddtables[i], i)
	end
	for catid, cats in ipairs(self.itemddtables) do
		for itemindex, itemddtable in ipairs(cats) do
			self:UpdateItemDropdownTableCats(itemddtable, catid, itemindex)
		end
	end
	self:UpdateTooltip()
end

function FuBar_ItemListFu:RemoveCat(catindex)
	for i = #self.db.char.data[catindex].items, 1, -1 do
		self:RemoveItem(catindex, i)
	end
	table.remove(self.catddtables, catindex)
	table.remove(self.db.char.data, catindex)
	for i = catindex, #self.db.char.data do
		self:UpdateCatDropdownTable(self.catddtables[i], i)
	end
	for catid, cats in ipairs(self.itemddtables) do
		for itemindex, itemddtable in ipairs(cats) do
			self:UpdateItemDropdownTableCats(itemddtable, catid, itemindex)
		end
	end
	self:UpdateTooltip()
end

function FuBar_ItemListFu:GetItemDropdownTable()
	local itemddtable = {
		type = "group",
		args = {
			cat = {
				type = "group",
				name = "Categorie",
				desc = "Change the item's categorie",
				args = {},
				order = 1
			},
			order = {
				type = "range",
				name = "Position",
				desc = "Change the items's position",
				min = 1,
				step = 1,
				order = 2
			},
			remove = {
				type = "execute",
				name = "Remove",
				desc = "Removes the item",
				order = 3
			}
		}
	}
	for catindex, catdata in ipairs(self.db.char.data) do
		local cattable = {
			type = "toggle",
			name = catdata.name,
			desc = string.format("Sets categorie to %s", catdata.name),
			isRadio = true,
			order = catindex
		}
		table.insert(itemddtable.args.cat.args, cattable)
	end
	return itemddtable
end

function FuBar_ItemListFu:UpdateItemDropdownTable(itemddtable, catindex, itemindex)
	itemddtable.name = (self.itemdata[self.db.char.data[catindex].items[itemindex]] and self.itemdata[self.db.char.data[catindex].items[itemindex]].name) or ("Item "..self.db.char.data[catindex].items[itemindex])
	itemddtable.desc = string.format("Settings for %s", itemddtable.name)
	itemddtable.icon = (self.itemdata[self.db.char.data[catindex].items[itemindex]] and self.itemdata[self.db.char.data[catindex].items[itemindex]].icon) or "Interface\\Icons\\INV_Misc_QuestionMark"
	itemddtable.order = itemindex
	for catid, data in ipairs(itemddtable.args.cat.args) do
		if catid == catindex then
			data.get = function() return true end
		else
			data.get = function() return false end
		end
		data.set = function() FuBar_ItemListFu:ChangeItemCat(catindex, itemindex, catid) end
	end
	itemddtable.args.cat.get = function() return itemddtable.name end
	itemddtable.args.cat.set = function(val) FuBar_ItemListFu:ChangeCatName(catindex, val) end
	itemddtable.args.order.min = #self.db.char.data[catindex].items <= 1 and 0 or 1
	itemddtable.args.order.max = #self.db.char.data[catindex].items
	itemddtable.args.order.get = function() return itemindex end
	itemddtable.args.order.set = function(val) FuBar_ItemListFu:ChangeItemPos(catindex, itemindex, val) end
	itemddtable.args.order.disabled = #self.db.char.data[catindex].items <= 1
	itemddtable.args.remove.func = function() FuBar_ItemListFu:RemoveItem(catindex, itemindex) end
end

function FuBar_ItemListFu:UpdateItemDropdownTableCats(itemddtable, catindex, itemindex)
	itemddtable.args.cat.args = {}
	for catid, catdata in ipairs(self.db.char.data) do
		local cattable = {
			type = "toggle",
			name = catdata.name,
			desc = string.format("Sets categorie to %s", catdata.name),
			set = function() FuBar_ItemListFu:ChangeItemCat(catindex, itemindex, catid) end,
			isRadio = true,
			order = catid
		}
		if catid == catindex then
			cattable.get = function() return true end
		else
			cattable.get = function() return false end
		end
		data.
		table.insert(itemddtable.args.cat.args, cattable)
	end
end

function FuBar_ItemListFu:ChangeItemCat(catindex, itemindex, catid)
	local itemid = table.remove(self.db.char.data[catindex].items, itemindex)
	local itemddtable = table.remove(self.itemddtables[catindex], itemindex)
	for i = itemindex, #self.itemddtables[catindex] do
		self:UpdateItemDropdownTable(self.itemddtables[catindex][i], catindex, i)
	end
	table.insert(self.db.char.data[catid].items, itemid)
	table.insert(self.itemddtables[catid], itemddtable)
	self:UpdateItemDropdownTable(itemddtable, catid, #self.itemddtables[catid])
	self:UpdateTooltip()
end

function FuBar_ItemListFu:ChangeItemPos(catindex, itemindex, pos)
	if pos == catindex then return end
	local itemid = table.remove(self.db.char.data[catindex].items, itemindex)
	table.insert(self.db.char.data[catindex].items, pos, itemid)
	local itemddtable = table.remove(self.itemddtables[catindex], itemindex)
	table.insert(self.itemddtables[catindex], pos, itemddtable)
	for i = itemindex, pos do
		self:UpdateItemDropdownTable(self.itemddtables[catindex][i], catindex, i)
	end
	self:UpdateTooltip()
end

function FuBar_ItemListFu:RemoveItem(catindex, itemindex)
	local itemid = table.remove(self.db.char.data[catindex].items, itemindex)
	if self.db.char.textitem == itemid then self.db.char.textitem = nil end
	table.remove(self.itemddtables[catindex], itemindex)
	for i = itemindex, #self.itemddtables[catindex] do
		self:UpdateItemDropdownTable(self.itemddtables[catindex][i], catindex, i)
	end
	self:UpdateTooltip()
end

